"use strict";!function(e,o){"object"==typeof exports&&"object"==typeof module?module.exports=o():"function"==typeof define&&define.amd?define([],o):"object"==typeof exports?exports.Airflow=o():(e.Airflow=e.Airflow||{},e.Airflow.materialIcons=o())}(self,()=>(self.webpackChunkAirflow=self.webpackChunkAirflow||[]).push([[186],{581:(e,o,f)=>{f.r(o)}},e=>{var o;return o=581,e(e.s=o)}]));